/*
 * PCB Exposure Timer, main.c
 *
 * (c) Axel (XL) Schwenke, axel.schwenke@gmx.net
 *
 * $Id: main.c 186 2012-10-30 08:41:52Z schwenke $
 *
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#include "hal.h"


/* global variables */

struct
{
    int16_t time;  /* seconds */
    uint8_t state; /* enum, see below */
} glob;

/* possible values for glob.state */
#define READY   0
#define RUNNING 1
#define DONE    2

/* display buffer, 5*8 bits, first 35 bits used */
uint8_t display[5];

/* 7 segment decoder table
 * 1=a, 2=b, 4=c ... 0x40=f */
uint8_t n_to_7[10] =
{
    0x3F, 0x06, 0x5B, 0x4F, 0x66, /* 0-4 */
    0x6D, 0x7D, 0x07, 0x7F, 0x6F  /* 5-9 */
};


/* sub routines */


/* all unused pins are inputs w/ pullup enabled */
void hw_init(void)
{
    PORTB = 0xFF;
    PORTD = 0xFF & ~_BV(bulb_pin);
    DDRB  = 0x00;
    DDRD  = (_BV(oe_pin) | _BV(clk_pin) | _BV(data_pin) | _BV(bulb_pin));
}


/* copy display buffer onto display */
void refresh(void)
{
    /* start data frame */
    clk_lo();
    oe_lo();

    /* send start bit */
    data_hi();
    wait();
    clk_hi();
    wait();
    clk_lo();

    /* followed by 35 data bits */
    for (uint8_t bit=0; bit<35; bit++) {
        if (display[bit/8] & (1<<(bit%8))) {
            data_hi();
        } else {
            data_lo();
        }
        wait();
        clk_hi();
        wait();
        clk_lo();
    }

    /* end data frame */
    oe_hi();
}


void setup_display(void)
{
    /* setup digits */
    uint16_t t = (uint16_t)glob.time;
    display[3] = n_to_7[t%10]; t /= 10;
    display[2] = n_to_7[t%6];  t /= 6;
    display[1] = n_to_7[t%10]; t /= 10;
    display[0] = n_to_7[t%10];

    /* turn on colon between minutes and seconds */
    display[1]  |= 0x80;    /* bit #16 */

    /* setup status LEDs */
    display[4] = 0;
    if (glob.state == READY) {
        display[4] |= 0x02; /* bit #34 */
    }
    if (glob.state == RUNNING) {
        display[4] |= 0x01; /* bit #33 */
    }
    if (glob.state == DONE) {
        display[0] |= 0x80; /* bit #8 */
    }
}


int __attribute__((OS_main)) main (void)
{
    hw_init();

    while (1) {
        glob.state = READY;
        setup_display();
        refresh();
        _delay_ms(100);

        glob.state = RUNNING;
        setup_display();
        refresh();
        _delay_ms(100);

        glob.state = DONE;
        setup_display();
        refresh();
        _delay_ms(100);

        glob.time = (glob.time == 100*60 ? 0 : glob.time+1);

    }
}

